#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

module TeXDist
    class Application
        class Options
            attr_accessor :distribution,:architecture,:user_domain,:index
        end
        class Parser
            def my_content(options)
                separator('Usage:')
                separator("    #{$me} activate [OPTIONS] -> activate a distribution or an architecture")
                separator('Options:')
                on('-d','--distribution DISTRIBUTION','the distribution to activate, active distribution when missing'){|a| options.distribution=a}
                on('-i','--index i','Select the ith bridge in the list.'){|n| n=n.to_i;options.index=n>0?n-1:n}
                separator('When neither an index or a name is given, the active distribution is chosen.')
                separator('Paths to bridges are accepted in the former.')
                separator('The ith bridge from the end is obtained with the negative index -i.')
                separator('Both options cannot be given at the same time.')
                on('-a','--architecture ARCHITECTURE','the architecture to activate in DISTRIBUTION'){|a| options.architecture=a}
                separator('Choose the architecture name "default" to best suit the hardware.')
                separator('At least one of the three above options is required.')
                on('-u','--user-domain','activate in the user domain'){options.user_domain=true}
                on('-s','--simulate','simulate activation'){options.simul8=true}
            end
        end
        def should_proceed?
            if setup_option_bridge
                if @bridge.available_archs.empty?
                    my_puts"No usable TeX distribution for #{@bridge.path}."
                elsif get_architecture.nil?
                    if @bridge.volume.my_startup?
                        return true if not @bridge.user_domain or is_user_domain?
                        my_puts 'Activate user defined bridges in the user domain only.'
                    else
                        my_puts 'Copy the bridge locally before activating a distribution not on the startup volume.'
                    end
                elsif @bridge.available_archs.index(get_architecture) or get_architecture=='default'
                    return true
                else
                    my_error "Architecture #{get_architecture} is not available."
                end
            end
            false
        end
        def proceed
            if get_architecture.nil?
                if is_user_domain?
                    my_log("Activating user only TeX distribution for #{@bridge.path}.",3)
                else
                    my_log("Activating TeX distribution for #{@bridge.path}",3)
                end
                Pathname.set_my_default_contents(@bridge,is_user_domain?,simul8?)
                my_puts active_bridge(is_user_domain?).corename_p.to_s unless active_bridge.nil?
            else
                my_log("Activating architecture #{get_architecture} for TeX distribution at #{@bridge.path}.",3)
                @bridge.activate_architecture(get_architecture,simul8?)
            end
            0
        rescue StandardError => e
            my_log(e,'Activation failed',3)
            my_puts 'You might need administrator privilege using sudo'
            1
        end
    end
end
