#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

module TeXDist
    class Application
        class Options
            attr_accessor :distributions,:bridge,:name,:bridge_version,:indices
            attr_accessor :root,:programs,:origin,:long,:user_domain
            attr_accessor :architecture,:engines,:available_archs,:commands
        end
        class Parser
            def my_content(options)
                separator('Usage:')
                separator("    #{$me} info [OPTIONS] -> get distribution bridge info")
                separator('Options for bridges')
                on('-d','--distribution DISTRIBUTION','get given DISTRIBUTION bridge info'){|x| options.distributions<<x}
                on('-i','--index i','Select the ith bridge in the result of the "list -we" command.'){|i| i.split(',').each{|n| n=n.to_i;n=n>0?n-1:n;options.indices<<n}}
                separator('When neither an index or a name is given, the active distribution takes place.')
                separator('Paths to bridges are accepted in the former as well as wildcard *.')
                separator('It can be used more than once.')
                separator('The latter accepts comma separated lists of indices and can be used more than once too.')
                separator('The ith bridge from the end is obtained with the negative index -i.')
                on('-b','--bridge','location of the bridge'){options.bridge=true}
                on('-n','--name','display short name?'){options.name=true}
                on('--bridge-version','display bridge version?'){options.bridge_version=true}
                on('-r','--root','display root?'){options.root=true}
                on('-p','--programs','display programs dir?'){options.programs=true}
                on('-o','--origin','location inside the bridge instead of real path'){options.origin=true}
                on('-a','--architecture','display active architecture'){options.architecture=true}
                on('-w','--which ENGINE','whether ENGINE is in the active architecture'){|es| options.engines.concat es.split(',')}
                on('--available-archs','display available archs'){options.available_archs=true}
                on('-l','--long','same as command line ls -lR. Overrides previous options.'){options.long=true}
                on('-u','--user-domain','active distribution in the user domain when requested.'){options.user_domain=true}
                separator('Other options')
                on('-t','--tex-binaries','List access to tex binaries.'){options.commands=true}
            end
        end
        def preprocess?
            @options.distributions=[]
            @options.indices=[]
            @options.engines=[]
            true
        end
        def show_info(b)
            if get_long
                d=b.contents_p
                d=d.symlink? ? d.parent+d.readlink: d
                puts `ls -lR #{d}`
                return
            end
            info=[]
            iinfo<<("bridge version #{b.version}") if is_bridge_version?
            info<<(b.root_p(get_origin.nil?)) if is_root?
            info<<(b.texbin_p(get_origin.nil?)) if is_programs?
            info<<((a=b.active_architecture) ? a: 'No active architecture...') if is_architecture?
            info<<(b.available_archs.join(',')) if is_available_archs?
            get_engines.each{|e| info << `#{$me_p} execute -d #{b.path} -c "which #{e}"`.chomp}
            info<<(is_name?? b.corename_p : b.path) if info.empty?
            puts info.join("\n")+"\n"
        end
        def proceed
            if is_commands?
               Pathname.my_print_texbin_info
            elsif get_indices.empty? and get_distributions.empty?
                if b=active_bridge(is_user_domain?)
                    puts 'Active distribution:'
                    show_info(b)
                end
            else
                option_bridges.each{|b| show_info(b)}
            end
            0
        end
        private :show_info
    end
end
