#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

module TeXDist
    class Application
        class Options
            attr_accessor :setup,:fix_me,:fix_texbin,:fix_factory,:fix_bridges,:local_domain,:user_domain
        end
        class Parser
            def my_content(options)
                separator('Usage:')
                separator("    #{$me} setup [OPTIONS] -> setup various things")
                separator('Options')
                on('-m','--fix-me','create a soft link to me in (...)/Library/TeX/Distributions/Programs/') do
                    options.setup=options.fix_me=true
                end
                on('-t','--fix-texbin','fix the various texbin soft links') do
                    options.setup=options.fix_texbin=true
                end
                on('-f','--fix-factory','recreate factory bridges') do
                    options.setup=options.fix_factory=true
                end
                on('-d','--distribution DISTRIBUTION','recreate factory bridge for DISTRIBUTION only') {|d| options.distribution=d
                    options.setup=true}
                on('-b','--fix-bridges','repairs distribution bridges internals') do
                    options.setup=options.fix_bridges=true
                end
                on('-a','--all','fix and repairs everything') do
                    options.setup=options.fix_texbin=options.fix_bridges=options.fix_factory=true
                end
                separator('At least one of the previous options is required')
                on('-s','--simulate','show which bridges are missing or symlink'){options.simul8=true}
                on('-u','--user-domain','texbin and bridges changes in the user domain only'){options.user_domain=true}
            end
        end
        def proceed
            (puts @parser;1) unless get_setup
            Pathname.my_fix_me(is_user_domain?,simul8?) if is_fix_me?
            Pathname.my_fix_texbin(is_user_domain?,simul8?) if is_fix_texbin?
            if is_fix_factory?
                Pathname.my_each_volume do |v|
                    my_log("Seeking for factory bridges on volume: #{v}",4)
                    v.each_bridge_name(false,true) do |f|
                        if b=Bridge.nouveau(f,false)
                            fext=f.basename.sub_ext($EXT)
                            n=b.path.parent.parent+fext
                            next if is_distribution? and not n.my_match?(get_distribution)
                            fc=b.path.my_contents
                            if @bridges.select{|b| b.volume==v and b.contents_target_p==fc and not b.user_domain}.empty?
                                my_puts("Missing bridge at #{f}") if simul8?
                                n.my_contents.my_make_symlink(fc.relative_path_from(n),simul8?)
                                puts Bridge.nouveau(n).path.to_s unless simul8?
                            else
                                my_log("Already there #{fext}",4)
                            end
                        end
                    end
                end
            end
            if is_fix_bridges?
                @bridges.select do |b|
                    is_user_domain? == b.user_domain
                end.each do |b|
                    b.fix_texbin(simul8?)
                end
            end
            0
        end
    end
end
