//
//  LTModel.h
//  Local TeX
//
//  Created by Richard Koch on 2/23/14.
//  Copyright (c) 2014 Richard Koch. All rights reserved.
//
// We create a Mutable Array of NSDictionaries. Each dictionary has  items,

//      Key                 Item
//      Active64            BOOL = YES for 64 bit, NO for 32 bit
//      PathTo32            Full Path to universal-darwin or null
//      PathTo64            Full Path to x86_64-darwub or null
//      PathToDist          Full Path to Distribution
//      Name                Name of Distribution

// We also create a dictionary containing
//
//      Key                 Item
//      ActiveName          Name of Active Distribution
//      MoreDists           Array of Other Distributions
//
// Here the structure of MoreDists is
//
//      Key                 Item
//      MoreName            Name of Distribution
//      MoreDistribution    Full Path To Distribution
//      MoreBinary          Full Path To Binary


#import <Foundation/Foundation.h>


@interface LTModel : NSObject
{
    
}

@property NSMutableArray         *Distributions;             // Distributions with TeXDist Structure
@property NSMutableDictionary    *extrasDictionary;          // Extra Distributions
@property NSString               *activeDistributionName;    // Active Distribution
@property BOOL                   changed;                    // User Default data changed

- (void)setupData;          // Setup Model
- (void)setupUserDefaults;  // Setup last two items from User Preferences
- (void)setupDistributions; // Setup Distributions
- (void)addExtraDistributionWithName:(NSString *)name binaryPath:(NSString *)binaryPath rootPath:(NSString *)rootPath;
- (void)removeExtraDistributionWithName:(NSString *)name;
- (void)writeDefaults;

- (BOOL)supports64Bits;
- (BOOL)activeDistributionAtPath: (NSString *)path;

@end
