//
//  LTModel.m
//  Local TeX
//
//  Created by Richard Koch on 2/23/14.
//  Copyright (c) 2014 Richard Koch. All rights reserved.
//

#import "LTModel.h"



@implementation LTModel

- (void)setupData
{
    [self setupUserDefaults];
    [self setupDistributions];
    self.changed = NO;
}

- (void)setupUserDefaults
{    
    NSUserDefaults  *myDefaults;
    NSDictionary    *defaultsDictionary;
    NSString        *defaultName;
    NSArray         *defaultMoreDists;
    NSMutableArray  *moreDists;
    
    
    myDefaults = [NSUserDefaults standardUserDefaults];
    defaultsDictionary = [myDefaults dictionaryForKey: @"localTeXExtrasData"];
    defaultName = [defaultsDictionary objectForKey:@"ActiveName"];
    defaultMoreDists = [defaultsDictionary objectForKey:@"MoreDists"];
    
    if (defaultsDictionary == nil)
        self.extrasDictionary = [NSMutableDictionary dictionaryWithCapacity:2];
    else
        self.extrasDictionary = [NSMutableDictionary dictionaryWithDictionary:self.extrasDictionary];
    
    if (defaultName == nil) {
        self.activeDistributionName = @"Unknown";
        [self.extrasDictionary setObject:self.activeDistributionName forKey:@"ActiveName"];
    }
    else
        self.activeDistributionName = defaultName;
    
    if (defaultMoreDists == nil) {
        moreDists = [NSMutableArray arrayWithCapacity:10];
        [self.extrasDictionary setObject:moreDists forKey:@"MoreDists"];
    }
    else
        [self.extrasDictionary setObject:defaultMoreDists forKey:@"MoreDists"];
}

- (void)setupDistributions
{
    NSFileManager *myFileManager;
    NSArray *LibraryTeXContents;
    NSMutableDictionary *ItemDictionary;
    NSArray *moreDistributions;
    NSUInteger numberOfEntries, i;
    NSString *itemPath;
    NSString *name;
    NSString *universalPath;
    NSString *x86Path;
    NSString *rootPath;
    BOOL universalOK, x86OK;
    BOOL supports64;
    NSString *moreDistPath, *moreBinaryPath, *moreName;
    
    myFileManager = [NSFileManager defaultManager];
    
    // ---------- First List Item -----------------------
    
    self.Distributions = [NSMutableArray arrayWithCapacity: 10];
    ItemDictionary = [NSMutableDictionary dictionaryWithCapacity: 5];
    if ([self supports64Bits])
        [ItemDictionary setObject: [NSNumber numberWithBool: YES] forKey: @"Active64"];
    else
        [ItemDictionary setObject: [NSNumber numberWithBool: NO] forKey: @"Active64"];
    [ItemDictionary setObject: [NSURL fileURLWithPath:@"/Library/TeX/Distributions/.DefaultTeX/Contents/Programs/texbin"]
                       forKey: @"PathTo32"];
    [ItemDictionary setObject: [NSURL fileURLWithPath:@"/Library/TeX/Distributions/.DefaultTeX/Contents/Root"]
                       forKey: @"PathToDist"];
    [ItemDictionary setObject: [NSURL fileURLWithPath:@"/Library/TeX/Distributions/.DefaultTeX/Contents/Programs/texbin"]
                       forKey: @"PathTo64"];
    [ItemDictionary setObject: @"Use Global Preference Pane"
                       forKey: @"Name"];
    [self.Distributions addObject: ItemDictionary];
    
    
    // --------- Other TexDist List Items -----------------
    
    LibraryTeXContents = [myFileManager contentsOfDirectoryAtPath: @"/Library/TeX/Distributions" error: NULL];
    if (LibraryTeXContents == nil)
        numberOfEntries = 0;
    else
        numberOfEntries = [LibraryTeXContents count];
    for (i = 0; i < numberOfEntries; i++) {
        itemPath = [LibraryTeXContents objectAtIndex:i];
        if ([[itemPath pathExtension] isEqualToString:@"texdist"]) {
            name = [itemPath stringByDeletingPathExtension];
            universalPath = [[@"/Library/TeX/Distributions/.FactoryDefaults/" stringByAppendingString: name]
                             stringByAppendingString:@"/Contents/Programs/i386"];
            x86Path = [[@"/Library/TeX/Distributions/.FactoryDefaults/" stringByAppendingString: name]
                       stringByAppendingString:@"/Contents/Programs/x86_64"];
            rootPath = [[@"/Library/TeX/Distributions/.FactoryDefaults/" stringByAppendingString: name]
                        stringByAppendingString:@"/Contents/Root"];
            // universalOK = [myFileManager fileExistsAtPath: [universalPath stringByAppendingString: @"/pdftex"]];
            // x86OK = [myFileManager fileExistsAtPath: [x86Path stringByAppendingString: @"/pdftex"]];
            universalOK = [self activeDistributionAtPath: universalPath];
            x86OK = [self activeDistributionAtPath: x86Path];
            supports64 = [self supports64Bits];
            if ((supports64 && x86OK) || universalOK)
            {
                ItemDictionary = [NSMutableDictionary dictionaryWithCapacity: 6];
                if (supports64 && x86OK)
                    [ItemDictionary setObject: [NSNumber numberWithBool: YES] forKey: @"Active64"];
                else
                    [ItemDictionary setObject: [NSNumber numberWithBool: NO] forKey: @"Active64"];
                if (universalOK)
                    [ItemDictionary setObject: [NSURL fileURLWithPath:universalPath]
                                       forKey: @"PathTo32"];
                if (x86OK)
                    [ItemDictionary setObject: [NSURL fileURLWithPath:x86Path]
                                       forKey: @"PathTo64"];
                [ItemDictionary setObject: [NSURL fileURLWithPath:rootPath]
                                   forKey: @"PathToDist"];
                [ItemDictionary setObject:name
                                   forKey: @"Name"];
                [self.Distributions addObject: ItemDictionary];
                
            }
        }
    }
    
    // -------- Add Distributions Defined By User ---------------
    moreDistributions = [self.extrasDictionary objectForKey: @"MoreDists"];
    numberOfEntries = 0;
    if (moreDistributions != nil)
        numberOfEntries = [moreDistributions count];
    if (numberOfEntries > 0)
        for (i = 0; i < numberOfEntries; i++) {
            
            moreName = [[moreDistributions objectAtIndex:i] objectForKey:@"MoreName"];
            moreBinaryPath = [[moreDistributions objectAtIndex:i] objectForKey:@"MoreBinary"];
            moreDistPath = [[moreDistributions objectAtIndex:i] objectForKey:@"MoreDistribution"];
            universalOK = [myFileManager fileExistsAtPath: [moreBinaryPath stringByAppendingString: @"/pdftex"]];
            if (universalOK) {
                ItemDictionary = [NSMutableDictionary dictionaryWithCapacity: 5];
                if (supports64)
                    [ItemDictionary setObject: [NSNumber numberWithBool: YES] forKey: @"Active64"];
                else
                    [ItemDictionary setObject: [NSNumber numberWithBool: NO] forKey: @"Active64"];
                [ItemDictionary setObject: [NSURL fileURLWithPath:moreBinaryPath]
                                   forKey: @"PathTo32"];
                [ItemDictionary setObject: [NSURL fileURLWithPath:moreBinaryPath]
                                   forKey: @"PathTo64"];
                [ItemDictionary setObject: [NSURL fileURLWithPath: moreDistPath]
                                   forKey: @"PathToDist"];
                [ItemDictionary setObject:moreName
                                   forKey: @"Name"];
                [self.Distributions addObject: ItemDictionary];
            }
        }
    
}

- (void)writeDefaults
{
    NSUserDefaults *myDefaults;

    if (self.changed) {
        myDefaults = [NSUserDefaults standardUserDefaults];
        [myDefaults removeObjectForKey:@"localTeXExtrasData"];
        [myDefaults setObject:self.extrasDictionary forKey:@"localTeXExtrasData"];
        [myDefaults synchronize];
        }
    
}

- (void)addExtraDistributionWithName:(NSString *)name binaryPath:(NSString *)binaryPath rootPath:(NSString *)rootPath;
{
    NSMutableArray          *moreDistributions;
    NSMutableDictionary     *extraItem;
    NSUInteger              index;
    
    
    moreDistributions = [NSMutableArray arrayWithArray:[self.extrasDictionary objectForKey:@"MoreDists"]];
    if (moreDistributions == nil)
        moreDistributions = [NSMutableArray arrayWithCapacity:1];
    extraItem = [NSMutableDictionary dictionaryWithCapacity: 3];
    [extraItem setObject: name forKey: @"MoreName"];
    [extraItem setObject: rootPath forKey: @"MoreDistribution"];
    [extraItem setObject: binaryPath forKey: @"MoreBinary"];
    index = [moreDistributions count];
    [moreDistributions setObject:extraItem atIndexedSubscript:index];
    [self.extrasDictionary removeObjectForKey:@"MoreDists"];
    [self.extrasDictionary setObject: moreDistributions forKey:@"MoreDists"];
    self.changed = YES;
    [self writeDefaults];

}

- (void)removeExtraDistributionWithName:(NSString *)name
{
    NSMutableArray          *moreDistributions;
    NSMutableDictionary     *extraItem;
    NSUInteger              size, i;
    NSString                *itemName;
    BOOL                    found;
    
    found = NO;
    moreDistributions = [NSMutableArray arrayWithArray:[self.extrasDictionary objectForKey:@"MoreDists"]];
    if (moreDistributions == nil)
        return;
    size = [moreDistributions count];
    for (i = 0; i < size; i++)
        {
            if (! found) {
            extraItem = [moreDistributions objectAtIndex:i];
            itemName = [extraItem objectForKey:@"MoreName"];
            if ([itemName isEqualToString:name]) {
                [moreDistributions removeObjectAtIndex: i];
                found = YES;
                }
            }
        }
    if (size > [moreDistributions count])
        {
            [self.extrasDictionary removeObjectForKey:@"MoreDists"];
            [self.extrasDictionary setObject: moreDistributions forKey:@"MoreDists"];
            self.changed = YES;
            [self writeDefaults];
        }

}

- (BOOL)supports64Bits
{ // Say Snow Leopard is 32 bits due to laziness
    
    if (NSAppKitVersionNumber >= NSAppKitVersionNumber10_7)
        return YES;
    else
        return NO;
}

- (BOOL)activeDistributionAtPath: (NSString *)path;
{
    NSString        *fullPath;
    NSFileManager   *myFileManager;
    
    myFileManager = [NSFileManager defaultManager];
    fullPath = [path stringByAppendingPathComponent: @"tex"];
    if ([myFileManager fileExistsAtPath:fullPath])
        return YES;
    fullPath = [path stringByAppendingPathComponent: @"pdftex"];
    if ([myFileManager fileExistsAtPath:fullPath])
        return YES;
    fullPath = [path stringByAppendingPathComponent: @"xetex"];
    if ([myFileManager fileExistsAtPath:fullPath])
        return YES;
    fullPath = [path stringByAppendingPathComponent: @"luatex"];
    if ([myFileManager fileExistsAtPath:fullPath])
        return YES;
    
    return NO;
    
}




@end
