//
//  TableControl.m
//  Local TeX
//
//  Created by Richard Koch on 2/16/14.
//  Copyright (c) 2014 Richard Koch. All rights reserved.
//

#import <CoreServices/CoreServices.h>
#import "TableControl.h"

@implementation TableControl

- (void)fillTable
{
    [myLTModel setupData];
    myLTModel.changed = NO;
    self.loaded = YES;
    [theTable reloadData];
}


- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
    if (self.loaded)
        return [myLTModel.Distributions count];
    else
        return 0;
}


- (NSView *)tableView:(NSTableView *)tableView
   viewForTableColumn:(NSTableColumn *)tableColumn
                  row:(NSInteger)row
{
    NSTableCellView *result;
    NSDictionary *theItem;
    BOOL is64Bit;
    NSString *theName;
    
    // Retrieve to get the @"MyView" from the pool or,
    // if no version is available in the pool, load the Interface Builder version
    
    // Set the stringValue of the cell's text field to the nameArray value at row
    //   if ([tableColumn.identifier isEqualToString: @"First"])
    
    theItem = [myLTModel.Distributions objectAtIndex: row];
    
    if ([tableColumn.identifier isEqualToString: @"First"])
            {
            result = [tableView makeViewWithIdentifier:@"MyTeXDistribution" owner:self];
            theName = [theItem objectForKey: @"Name"];
            result.textField.stringValue = theName;
            if ([theName isEqualToString: myLTModel.activeDistributionName]) {
                [theTable selectRowIndexes: [NSIndexSet indexSetWithIndex: row] byExtendingSelection: NO];
            }
        }
    else if ([tableColumn.identifier isEqualToString: @"Second"])
            {
                result = [tableView makeViewWithIdentifier:@"MyProcessor" owner:self];
                is64Bit = [[theItem objectForKey: @"Active64"] boolValue];
                if (is64Bit)
                    result.textField.stringValue = @"Intel 64";
                else
                    result.textField.stringValue = @"Universal";
            }

    return result;
}


- (void)tableViewSelectionDidChange:(NSNotification *)aNotification;
{
    NSDictionary *theItem;
    NSURL *theURL;
    NSFileManager *theFileManager;
    NSString* theLinkPath;
    NSString* theName;
    NSString* oldName;
    
    oldName = myLTModel.activeDistributionName;
    
    NSInteger theRow = [theTable selectedRow];
    if (theRow < 0) {
        myLTModel.activeDistributionName = @"Unknown";
       // theFileManager = [NSFileManager defaultManager];
       // theLinkPath = [@"~/Library/TeX/texbin" stringByExpandingTildeInPath];
       // [theFileManager removeItemAtPath: theLinkPath error: NULL];
       // theLinkPath = [@"~/Library/TeX/texRoot" stringByExpandingTildeInPath];
       //  [theFileManager removeItemAtPath: theLinkPath error: NULL];
         }
    if (theRow >= 0)
        {
        theItem = [myLTModel.Distributions objectAtIndex: theRow];
        if ([theItem objectForKey:@"Active64"])
            theURL = [theItem objectForKey: @"PathTo64"];
        else
            theURL = [theItem objectForKey: @"PathTo32"];
        theLinkPath = [@"~/Library/TeX/texbin" stringByExpandingTildeInPath];
        theFileManager = [NSFileManager defaultManager];
        [theFileManager removeItemAtPath: theLinkPath error: NULL];
        [theFileManager createSymbolicLinkAtPath: theLinkPath withDestinationPath: [theURL path] error: NULL];
        
        theLinkPath = [@"~/Library/TeX/texRoot" stringByExpandingTildeInPath];
        theURL = [theItem objectForKey: @"PathToDist"];
        [theFileManager removeItemAtPath: theLinkPath error: NULL];
        [theFileManager createSymbolicLinkAtPath: theLinkPath withDestinationPath: [theURL path] error: NULL];
        
        theName = [theItem objectForKey: @"Name"];
        myLTModel.activeDistributionName = theName;
        [myLTModel.extrasDictionary removeObjectForKey:@"ActiveName"];
        [myLTModel.extrasDictionary setObject: theName forKey:@"ActiveName"];
        }
    
    if (! [oldName isEqualToString:myLTModel.activeDistributionName])
        myLTModel.changed = YES;
        [myLTModel writeDefaults];
 }



- (IBAction)addDistribution:(id)sender
{
    
        [NSApp beginSheet:newDistributionPanel
           modalForWindow:[theTable window]
            modalDelegate:self
           didEndSelector:nil
              contextInfo:nil];
    
}

- (IBAction)removeDistribution:(id)sender
{
    [myRemoveDistTable runRemoveDistributionPanel];
}



- (IBAction)okPushed:(id)sender
{
    NSString *newDistributionName;
    NSString *newDistributionPath;
    NSString *newBinaryPath;
    
    newDistributionName = [distributionName stringValue];
    newDistributionPath = [distributionPath stringValue];
    newBinaryPath = [binaryPath stringValue];
    
    [NSApp endSheet: newDistributionPanel];
    [newDistributionPanel orderOut:sender];
    
    // myFileManager = [NSFileManager defaultManager];
    //if ([myFileManager fileExistsAtPath: [newBinaryPath stringByAppendingString: @"/pdftex"]])
    if ([myLTModel activeDistributionAtPath:newBinaryPath])
        {
        [myLTModel addExtraDistributionWithName:newDistributionName binaryPath: newBinaryPath rootPath:newDistributionPath];
        [myLTModel setupData];
        [self setLoaded: YES];
        [theTable reloadData];
        }
    else
        [self errorBinaryDirectory];
    
}

- (IBAction)cancelPushed:(id)sender
{
    [NSApp endSheet: newDistributionPanel];
    [newDistributionPanel orderOut:sender];
}

- (void)errorTeXDirectory
{
    NSAlert *myAlert;
    
    myAlert = [NSAlert alertWithMessageText:@"Cannot Create TeX Folder" defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:@"~/Library/TeX exists as a file, so the TeX folder cannot be created."];
    [myAlert beginSheetModalForWindow:[theTable window] modalDelegate: self didEndSelector: nil contextInfo: nil];
}

- (void)errorBinaryDirectory
{
    NSAlert *myAlert;
    
    myAlert = [NSAlert alertWithMessageText:@"Cannot Create New Distribution" defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:@"The proposed binary directory does not contain tex, pdftex, luatex, or xetex."];
    [myAlert beginSheetModalForWindow:[theTable window] modalDelegate: self didEndSelector: nil contextInfo: nil];
}

- (void)registerForDrags
{
    [distributionPath registerForDraggedTypes:[NSArray arrayWithObjects:
                                               NSURLPboardType, nil]];
    [binaryPath registerForDraggedTypes: [NSArray arrayWithObjects:
                                                NSURLPboardType, nil]];
}

@end
