#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

module TeXDist
    class Application
        class Options
            attr_accessor :every,:main,:path,:user_domain,:copyable,:show_index,:indices
        end
        class Parser
            def my_content(options)
                separator('Usage:')
                separator("    #{$me} list [OPTIONS] -> list available TeX distributions")
                separator('Options')
                on('-e','--every','list every bridges, whether there is a suitable distribution ot not') {options.every=true}
                on('-a','--activable','list only activable bridges') {options.main=true}
                on('-i','--index i','Select the ith bridge in the whole list'){|i| i.split(',').each{|n| n=n.to_i;n=n>0?n-1:n;options.indices<<n}}
                separator('Accepts comma separated lists of indices. Can be used more than once.')
                separator('The ith bridge from the end is obtained with the negative index -i.')
                on('-s','--show-index','Show the distribution index'){options.show_index=true}
                on('-p','--path','location of the bridge'){options.path=true}
                on('-c','--copyable','Select bridges that can be copied'){options.copyable=true}
                on('-u','--user-domain','only bridges of the user domain'){options.user_domain=true}
            end
        end
        def preprocess?
            @options.indices=[]
            true
        end
        def info(b)
            "#{(get_path.nil? ? b.corename_p: b.path)}"
        end
        def proceed
            if get_indices.empty?
                @bridges.select do |b|
                    is_every? or (b.has_arch? and (! is_main? or b.main))
                end.select do |b|
                    not is_user_domain? or b.user_domain
                end.select do |b|
                    not is_copyable? or b.version>1
                end.each do |b|
                    out=info(b)+"\n"
                    out="#{@bridges.index(b)+1}:"+out if is_show_index?
                    puts out
                end
            else
                get_indices.uniq.sort.each do |i|
                    b=@bridges[i]
                    puts info(b) if b
                end
            end
            0
        end
    end
end
