#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

require 'fileutils'
require 'tmpdir'

module TeXDist
    class Application
        class Helper
            attr_accessor :base,:distributions,:root
            def copy_and_simplify(from,to_dir,simul8=false,depth=nil)
                raise ArgumentError if not from or not to_dir
                to=to_dir+from.basename
                to.my_force_delete(simul8)
                my_log("Copy: #{from} to #{to}",3)
                my_log("Working directory: #{depth}",1)
                if from.symlink?
                    my_log('Symbolic link',2)
                    p=Pathname.new(from.readlink)
                    depth=Pathname.new('.') if depth.nil?
                    my_log("Working directory: #{depth}",1)
                    if r=p.my_relative_path_from(depth,false)
                        if r=r.my_relative_path_from(@base,false)#p=depth+@base+r
                            my_log("Relative from base: #{r}",1)
                            unless rr=r.my_relative_path_from(@distributions)
                                target=depth+'Base'+r
                            else#p=depth+@base+@distributions+rr
                                my_log("Relative from distributions: #{rr}",1)
                                unless r=rr.my_relative_path_from(@root)
                                    target=depth+'Distributions'+rr
                                else#p=depth+@base+@distributions+@root+r
                                    my_log("Relative from root: #{r}",1)
                                    target=depth+'Root'+r
                                end
                            end
                            my_log("Target: #{target}",1)
                            to.my_make_symlink(target,simul8)
                            my_log("Symbolic link copied translated #{to} to #{to.readlink}",3) unless simul8
                            return
                        end
                    end
                    if simul8
                        my_log("Copy #{from} to #{to}",3)
                    else
                        FileUtils.copy_entry(from.to_s,to.to_s)
                    end
                    my_log("Symbolic link copied as is #{to} -> #{to.readlink}",3) unless simul8
                elsif from.directory?
                    my_log('Directory...',1)
                    to.my_mkpath(simul8)
                    depth=depth.nil? ? Pathname.new('.'): depth+'..'
                    from.each_child{|c| copy_and_simplify(c,to,simul8,depth)}
                elsif from.file?
                    my_log('File...',1)
                    if simul8
                        my_puts("Copy #{from} to #{to}")
                    else
                        FileUtils.copy_entry(from.to_s,to.to_s)
                    end
                else
                    raise 'An error occurred (code 174)'
                end
                my_log('Done',1)
            end
            def my_rename(tmp,to,simul8)
                final=to+'Contents'
                intermediate=to+'Contents.converted'
                intermediate.my_force_delete(simul8)
                tmp.my_contents.my_rename(intermediate,simul8)
                final.my_force_delete(simul8)
                intermediate.my_rename(final,simul8)
                true
            end
            def complete_upgrade(p,tmp,simul8)
                tmp.my_contents.my_base.my_make_symlink(base,simul8)
                my_rename(tmp,p.parent,simul8)
            end
            def complete_copy(from,to,tmp,simul8)
                c=to.my_contents
                c.my_force_delete(simul8)
                c.my_mkpath(simul8)
                base=(from.my_contents+@base).my_realpath.my_relative_path_from(to.my_contents,false)
                abort "Copy failed" unless base
                tmp.my_contents.my_base.my_make_symlink(base,simul8)
                my_rename(tmp,to,simul8)
            end
            def duplicate(p,simul8)
                r=p.my_root.readlink.cleanpath(true)
                return false if r.absolute?
                @base,@distributions,@root=r.my_badistro
                tmp = Pathname.new(Dir.mktmpdir)
                copy_and_simplify(p,tmp,simul8)
                t=tmp.my_contents
                t.my_distributions.my_make_symlink(Pathname.new('Base')+@distributions,simul8)
                t.my_root.my_make_symlink(Pathname.new('Distributions')+@root,simul8)
                tmp
            rescue StandardError => e
                tmp.my_force_delete if tmp
                raise $!, "#{$!} (duplicate failed)", $!.backtrace
            end
            def upgrade(bridge_path,simul8=false)
                p=bridge_path.my_contents
                raise ArgumentError if not p.exist?
                my_log("Upgrading #{bridge_path}",4)
                p=p.realpath
                if tmp=duplicate(p,simul8)
                    return complete_upgrade(p,tmp,simul8)
                end
            rescue StandardError => e
                my_log(e,'To upgrade global bridges, you might need administrator privilege using sudo',5)
            ensure
                tmp.my_force_delete if tmp
                false
            end
            def copy(bridge_path,to,simul8=false)
                p=bridge_path.my_contents
                raise ArgumentError if not p.exist?
                my_log("Copy #{bridge_path} to #{to}",4)
                p=p.realpath
                if tmp=duplicate(p,simul8)
                    return complete_copy(bridge_path,to,tmp,simul8)
                end
            rescue StandardError => e
                my_log(e,'To copy global bridges, you might need administrator privilege using sudo',5)
            ensure
                tmp.my_force_delete if tmp
                false
            end
            private :duplicate,:complete_upgrade,:complete_copy,:copy_and_simplify
        end
    end
end
