#!/usr/bin/env ruby

raise "Ruby 1.9.3 required" if ("%03d%03d%03d" % (RUBY_VERSION.split('.')<<0<<0)).to_i < 1009003

$texdistversion='2.0'

require 'pathname'
$me=Pathname.new(__FILE__).basename
$me_p=Pathname.new(__FILE__)

require 'logger'
$logger=Logger.new(STDOUT)
$logger.formatter=proc do |severity, datetime, progname, msg|
    "#{$me}-> #{msg}\n"
end
$MaxLogLevel=5
$logger.level=$MaxLogLevel

class Object
    def my_log(e,msg=nil,level=2)
        if msg.class <= Fixnum
            level=msg
            msg=nil;
        end
        if not e.nil? and not e.class <= Exception
            msg=e.to_s
            e=nil;
        end
        $logger.add(level,msg.to_s) if not msg.nil? and not msg.to_s.empty?
        unless e.nil?
            $logger.add(level,e.message)
            e.backtrace.each do |l|
                $logger.add([0,level-1].max,l)
            end
        end
    end
    def my_puts(msg)
        my_log(nil,msg,$MaxLogLevel)
    end
    def my_error(msg)
        my_puts('Error: '+msg)
    end
end
class NilClass
    def empty?
        true
    end
end
require 'optparse'
class OptionParser
    def my_header
        separator("    #{$me} is a command line utility to activate TeX distributions and")
        separator('    more generally manage TeX distribution bridges,')
        separator('    id est bundles with a texdist path extension.')
        self
    end
    def my_trailer
        self.on('-h','--help','show this help and quit') do
            puts self
            exit
        end
        self.on('-v LEVEL','--verbosity LEVEL',Numeric,"set verbosity level (0..#{$MaxLogLevel-1})") do |option|
            $logger.level=$MaxLogLevel-[[option,0].max,$MaxLogLevel].min
        end
        self.on('--version','show version number and quit') do
            puts $texdistversion
            exit
        end
        self
    end
    def my_signature
        separator('© 2014 jerome.laurens@u-bourgogne.fr')
        separator('Disclaimer: This software is provided "as is" and without any express')
        separator('   or implied warranties, including, without limitation, the implied')
        separator('   warranties of merchantibility and fitness for a particular purpose.')
        self
    end
end

$parser=OptionParser.new do |o|
    o.banner='Presentation:'
    o.my_header
    o.separator('Usages:')
    o.separator("    #{$me} [ACTION] [OPTIONS] ...")
    o.separator('Available actions:')
    o.separator('    list:     list available TeX distributions')
    o.separator('    info:     display TeX distribution bridge info')
    o.separator('    activate: activate TeX distribution')
    o.separator('    copy:     copy TeX distribution bridges')
    o.separator('    delete:   delete TeX distribution bridges')
    o.separator('    upgrade:  upgrade TeX distribution bridges')
    o.separator('    execute:  to execute commands')
    o.separator('    setup:    setup, fix or repair')
    o.separator('Common options:')
    o.my_trailer
    o.separator('Dedicated help on ACTION with')
    o.separator('    texdist ACTION --help')
    o.separator('Examples,')
    o.separator('    please, see TeXdist description.')
    o.my_signature
end
$action=ARGV.shift
(puts $parser;exit 1) if $action.nil?
if $action.match(/-.+/)
    ARGV.unshift($action)
    begin
        $parser.parse!
    rescue
        puts $parser
        exit 1
    end
    puts $parser
    exit 0
end
require_relative 'lib/texdist'
$main=TeXDist::Application.new
$main.run($action)
