#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

$PATH_PREFIX='/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:'

module TeXDist
    class Application
        class Options
            attr_accessor :commands,:searchs,:index
        end
        class Parser
            def my_content(options)
                separator('Usage:')
                separator("    #{$me} execute [OPTIONS] -> execute commands in the context of a distribution")
                separator('Options:')
                on('-d','--distribution DISTRIBUTION','use the given DISTRIBUTION instead of the active one'){|d| options.distribution=d}
                on('-i','--index i','use the ith bridge in the list.'){|i| options.index=i}
                on('-c','--command COMMAND','execute COMMAND in the context of DISTRIBUTION,'){|c| options.commands<<c}
                on('-w','--where WHAT','find WHAT (use kpsewhich),'){|ss| options.searchs.concat ss.split(',')}
                separator('Both previous options can be used more than once.')
                separator('The latter accepts a comma separated list of terms.')
                on('-s','--simulate','display the PATH and return'){options.simul8=true}
            end
        end
        def preprocess?
            @options.commands=[]
            @options.searchs=[]
            true
        end
        def should_proceed?
            if get_commands.empty? and get_searchs.empty?
                my_puts "Nothing to execute"
            elsif setup_option_bridge
                my_log "Using TeX distribution #{get_distribution}",3
                return true if @bridge.texbin_p(true)
                abort "Distribution bridge #{@bridge.corename_p} has no active architecture."
            end
            false
        end
        def proceed
            if get_commands
                p=@bridge.texbin_p(true).to_s
                my_log "Appending #{p} to stantard PATH",2
                out=0
                get_commands.each do |c|
                    if simul8?
                        puts "Will execute \"#{c}\" with PATH=\"#{$PATH_PREFIX+p}\"\n"
                    else
                        begin
                            old=ENV['PATH']
                            ENV['PATH']=$PATH_PREFIX+p
                            system c
                        ensure
                            ENV['PATH']=old
                        end
                        return out if out=$?.exitstatus
                    end
                end
            end
            unless get_searchs.empty?
                ss=get_searchs.join(' ')
                which=@bridge.programs_p+@bridge.preferred_bin+"kpsewhich"
                if simul8?
                    puts "Will search \"#{ss}\" with #{which}/\n"
                else
                    system "#{which} #{ss}"
                    return out if out=$?.exitstatus
                end
            end
            out
        end
    end
end
