#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

require_relative 'pathname'
require_relative 'bridge'

module TeXDist
    class Application
        class Helper
            def Helper.version
                2
            end
            def Helper.upgrade(path,simul8=false)
                require_relative 'helper'
                Helper.new.upgrade(path,simul8)
            end
            def Helper.copy(path,to,simul8=false)
                require_relative 'helper'
                Helper.new.copy(path,to,simul8)
            end
        end
        class Options
            attr_accessor :help,:simul8,:distribution
        end
        class Parser<OptionParser
            def my_content
                raise NotImplementedError
            end
        end
        attr_accessor :bridges,:bridge,:options,:parser
        def method_missing(meth, *args, &block)
            if m=/^is_(.*)\?$/.match(meth.to_s) and @options.respond_to?(m[1].to_sym)
                self.class.send :define_method, meth do
                    @options.send(m[1])==true
                end
                self.send(meth)
            elsif m=/^get_(.*)$/.match(meth.to_s) and @options.respond_to?(m[1].to_sym)
                self.class.send :define_method, meth do
                    @options.send(m[1])
                end
                self.send(meth)
            else
                super
            end
        end
        def respond_to?(meth)
            if m=/^is_(.*)\?$/.match(meth.to_s) and @options.respond_to?(m[1].to_sym)
                true
            elsif m=/^get_(.*)$/.match(meth.to_s) and @options.respond_to?(m[1].to_sym)
                true
            else
                super
            end
        end
        def simul8?
            is_simul8?
        end
        def get_bridges(name=nil)
            return [active_bridge] if name.nil?
            return @bridges if name=='*'
            r=@bridges.select{|b| b.path.my_match?(name)}
            return r if r.count>0
            r=@bridges.select{|b| b.contents_target_p.my_match?(name)}
            return r if r.count>0
            n=Pathname.pwd+name
            b=Bridge.nouveau(n,false)
            b.nil? ? []:[b]
        end
        def option_bridges
            unless @option_bridges
                @option_bridges=[]
                bs=@bridges.values_at(*get_indices)
                bs.compact!
                get_distributions.each{|n| bs.concat get_bridges(n)}
                @option_bridges=bs.uniq.sort
            end
            @option_bridges
        end
        def setup_option_bridge
            bs=[]
            bs.concat get_bridges(get_distribution) if get_distribution
            if bs.count>1
                my_error "Too many bridges named #{get_distribution}."
                false
            elsif get_index
                bs<<bridges[get_index.to_i]
                bs.compact!
                bs.uniq!
                if bs.count>1
                    my_error "Given index is not compatible with given distribution name."
                    false
                else
                    @bridge=bs.first
                end
            else
                bs=get_bridges if bs.empty?
                @bridge=bs.first
            end
        end
        def active_bridge(user_domain=false)
            d=Pathname.get_my_default_contents(user_domain)
            unless d.nil?
                bbs=get_bridges(d.parent)
                return bbs.first
            end
            nil
        end
        def preprocess?
            true
        end
        def should_proceed?
            true
        end
        def run(action)
            begin
                require_relative 'actions/'+action
            rescue LoadError
                my_error("Unknown action #{action}")
                system("#{$me_p} -h")
                return 1
            end
            begin
                @options=Options.new
                @parser=Parser.new do |o|
                    o.banner='Presentation:'
                    o.my_header
                    o.my_content(@options)
                    o.my_trailer
                    o.my_signature
                end
                return (puts @parser;1) unless preprocess?
                @parser.parse!
            rescue SystemExit => e
                return 0
            rescue StandardError => e
                my_log(e,'Operation failed!',4)
                puts @parser
                return 1
            end
            my_log "action is #{action}",4
            return(puts @parser;0) if is_help?
            my_puts 'Simulation mode' if simul8?
            my_log("run with bridge repositories\n"+Pathname.my_each_volume.map{|v| v.bridges_repository.to_s}.join("\n"),3)
            @bridges=[]
            Pathname.my_each_volume do |v|
                v.each_bridge_name do |n|
                    b=Bridge.nouveau(n)
                    if b
                        f=@bridges.select do |bb|
                            bb.contents_target_p==b.contents_target_p
                        end.first
                        if f.nil?
                            b.principal=true
                        elsif b.basename_p.to_s<f.basename_p.to_s
                            b.principal=true
                            f.principal=false
                        else
                            b.principal=false
                            f.principal=true
                        end
                        @bridges<< b
                        @bridges.sort!
                    end
                end
            end
            Pathname.my_home.each_bridge_name do |n|
                b=Bridge.nouveau(n)
                if b
                    b.user_domain=true
                    @bridges.each do |bb|
                        if bb.main and bb.volume!=Pathname.my_root and bb.root_p(true)==b.root_p(true)
                            bb.main=false
                        end
                    end
                    @bridges<< b
                    @bridges.sort!
                end
            end
            unless action=="activate"
                b=active_bridge
                if b
                    my_error "No active architecture (run command \"#{$me_p} activate -a default\")." unless b.texbin_p(true)
                else
                    my_error "No active distribution (run commands \"#{$me_p} list -s\" and \"#{$me_p} activate -i ...\")."
                end
            end
            if should_proceed?
                my_log("Will proceed #{action}",3)
                proceed
                my_log("Did proceed #{action}",3)
            end
        rescue StandardError => e
            my_puts 'Operation failed: you might need administrator privilege.'
            my_log(e,3)
            return 1
        end
    end
end
