#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

module TeXDist
    class Application
        attr_accessor :bridge,:destination
        class Options
            attr_accessor :rename,:user_domain,:relative,:index
        end
        class Parser
            def my_content(options)
                separator('Usage:')
                separator("    #{$me} copy [Options] -> make a copy of a bridge")
                separator('Options:')
                on('-b','--bridge DISTRIBUTION','will be copied. Defaults to the active bridge.'){|x| options.distribution=x}
                on('-i','--index i','Copy the ith bridge in the list.'){|n| n=n.to_i;options.index=n>0?n-1:n}
                separator('When neither an index or a name is given, the active distribution is chosen.')
                separator('Paths to bridges are accepted in the former.')
                separator('The ith bridge from the end is obtained with the negative index -i.')
                separator('Both options cannot be given at the same time.')
                on('-n','--new-name NAME','change the name of the copy to NAME.'){|x| options.rename=x}
                on('-u','--user-domain','copy in the user domain.'){options.user_domain=true}
                on('-r','--relative','use relative paths.'){options.relative=true}
                on('-s','--simulate','simulate copy.'){options.simul8=true}
            end
        end
        def should_proceed?
            if setup_option_bridge
                name=get_rename
                if name.empty?
                    name=@bridge.path.basename
                    v=@bridge.volume
                    name.concat "(#{v})" if v.to_s.length>1
                end
                @destination=(Pathname.my_root(is_user_domain?).bridges_repository+name).sub_ext($EXT)
                return true unless @destination.exist?
                my_error "#{destination} destination already exists."
            end
            false
        end
        def proceed
            my_log("#{bridge.corename_p} copied",5) if Helper.copy(bridge.path,destination,simul8?) and not simul8?
            0
        end
    end
end
